﻿//
// Copyright (c) 2015, MindFusion LLC - Bulgaria.
//

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using MindFusion.Scheduling.Reporting;


namespace MindFusion.Scheduling.Wpf.Samples.CS.Report
{
	/// <summary>
	/// Interaction logic for MainWindow.xaml
	/// </summary>
	public partial class MainWindow : Window
	{
		public MainWindow()
		{
			InitializeComponent();

			calendar.BeginInit();

			for (int i = 0; i < 2; i++)
				calendar.TimetableSettings.Dates.Add(DateTime.Today.AddDays(i + 1));

			calendar.ItemContacts.Add(
				new Contact()
				{
					FirstName = "Norman",
					LastName = "Wright",
					Name = "Norman Wright",
				});
			calendar.ItemContacts.Add(
				new Contact()
				{
					FirstName = "John",
					LastName = "Nelson",
					Name = "John Nelson",
				});
			calendar.ItemContacts.Add(
				new Contact()
				{
					FirstName = "Mark",
					LastName = "Parker",
					Name = "Mark Parker",
				});

			calendar.GroupType = GroupType.GroupByContacts;
			calendar.TimetableSettings.SubHeaderSize = 32;

			calendar.EndInit();

			// Create the report
			report = new MindFusion.Scheduling.Reporting.Report(calendar.Schedule);

			ReportColumn column = report.CreateColumn();
			column.Property = "HeaderText";
			column.Header = "Appointment";
			column.Width = 100;

			column = report.CreateColumn();
			column.Property = "StartTime";
			column.Header = "From";
			column.Width = 40;
			column.Sort = Sort.Ascending;
			column.SortPriority = 2;
			column.ContentStyle.Background = new SolidColorBrush(Color.FromArgb(255, 245, 245, 255));
			column.AlternativeContentStyle.Background = new SolidColorBrush(Color.FromArgb(255, 230, 230, 255));

			column = report.CreateColumn();
			column.Property = "Duration";
			column.Header = "Duration";
			column.Width = 20;

			report.QueryText += new EventHandler<ReportQueryTextEventArgs>(report_QueryText);
			report.QueryStyle += new EventHandler<ReportQueryStyleEventArgs>(report_QueryStyle);

			report.Options.HeaderStyle.HorizontalContentAlignment = AlignmentX.Left;
			report.Options.HeaderStyle.Background = new LinearGradientBrush(
				new GradientStopCollection(
					new GradientStop[]
					{
						new GradientStop(Color.FromArgb(255, 240, 240, 240), 0),
						new GradientStop(Color.FromArgb(255, 192, 192, 192), 1),
					}), 90);
			report.Options.HeaderStyle.TopBorderBrush = Brushes.WhiteSmoke;
			report.Options.HeaderStyle.TopBorderThickness = 1;
			report.Options.HeaderStyle.LeftBorderBrush = Brushes.WhiteSmoke;
			report.Options.HeaderStyle.LeftBorderThickness = 1;
			report.Options.HeaderStyle.BottomBorderBrush = Brushes.Gray;
			report.Options.HeaderStyle.BottomBorderThickness = 1;
			report.Options.HeaderStyle.RightBorderBrush = Brushes.Gray;
			report.Options.HeaderStyle.RightBorderThickness = 1;

			report.Options.ContentStyle.LeftBorderBrush = Brushes.WhiteSmoke;
			report.Options.ContentStyle.LeftBorderThickness = 1;
			report.Options.ContentStyle.RightBorderBrush = Brushes.LightGray;
			report.Options.ContentStyle.RightBorderThickness = 1;

			report.Options.GroupHeaderStyle.Background = new SolidColorBrush(Colors.LightGray);
			report.Options.GroupHeaderStyle.HorizontalContentAlignment = AlignmentX.Center;

			report.Options.AbsoluteColumnWidth = false;
			report.Options.GroupType = ReportGroupType.ByContacts;
			report.Options.DocumentName = "MindFusion.Scheduling Test Report";
			report.Options.HeaderSpacing = 0;
		}

		void report_QueryStyle(object sender, ReportQueryStyleEventArgs e)
		{
			if (e.Column.Property == "Duration" && customPaintDuration)
			{
				TimeSpan duration = e.Item.EndTime - e.Item.StartTime;
				if (duration > TimeSpan.FromHours(4))
				{
					e.Style = new ReportStyle()
					{
						Background = new SolidColorBrush(Color.FromArgb(100, 255, 0, 0)),
					};
				}
			}
		}

		private void report_QueryText(object sender, ReportQueryTextEventArgs e)
		{
			if (e.Column.Property == "Duration")
			{
				TimeSpan duration = e.Item.EndTime - e.Item.StartTime;
				e.Representation = string.Format("{0} hours", duration.TotalHours);
			}
			else if (e.Column.Property == "StartTime")
			{
				e.Representation = ((DateTime)e.Value).ToString("d, MMM 'at' H:mm");
			}
		}

		private void _menuItemPreview_Click(object sender, EventArgs e)
		{
			report.PrintPreview(DateTime.MinValue, DateTime.MaxValue);
		}

		private void _menuItemPrint_Click(object sender, EventArgs e)
		{
			report.Print(DateTime.MinValue, DateTime.MaxValue);
		}

		private void _menuItemCreateAppointments_Click(object sender, RoutedEventArgs e)
		{
			string[] titles = new string[]
			{
				"Visit the dentist",
				"Meet Laura",
				"Drive the kids to school",
				"Lunch with the colleagues",
				"Laundry time",
				"Dinner at the club",
				"Get the mother-in-law from the airport",
				"Pay a visit to the fitter",
				"Breakfast at the beach",
				"Call the neighbours",
				"Work meeting",
				"Walk about the central park",
				"Vacation"
			};
			Random random = new Random((int)DateTime.Now.Ticks);

			calendar.BeginInit();
			for (int i = 0; i < 20; i++)
			{
				Contact contact = calendar.ItemContacts[random.Next() % calendar.ItemContacts.Count];

				Appointment app = new Appointment();
				app.StartTime = calendar.TimetableSettings.Dates[0].Date.AddHours(random.Next() % (calendar.TimetableSettings.Dates.Count * 24));
				app.EndTime = app.StartTime.AddHours(1 + random.Next() % 5);
				app.Contacts.Add(contact);
				app.HeaderText = titles[random.Next() % titles.Length];
				calendar.Schedule.Items.Add(app);
			}
			calendar.EndInit();
		}

		private void _menuItemOptions_Click(object sender, RoutedEventArgs e)
		{
			OptionsWindow options = new OptionsWindow();

			options.enableGrouping.IsChecked = report.Options.GroupType != ReportGroupType.None;
			options.customPaint.IsChecked = customPaintDuration;
			options.displayDuration.IsChecked = report.Columns[2].Visible;

			int sortColumnA = 0;
			int sortColumnB = 0;
			for (int i = 0; i < report.Columns.Count; i++)
			{
				if (report.Columns[i].Sort != Sort.None)
				{
					if (report.Columns[i].SortPriority == 2)
						sortColumnA = i + 1;
					else
						sortColumnB = i + 1;
				}
			}

			switch (sortColumnA)
			{
				case 0:
					options.sort0.IsChecked = true;
					break;

				case 1:
					options.sort1.IsChecked = true;
					break;

				case 2:
					options.sort2.IsChecked = true;
					break;

				case 3:
					options.sort3.IsChecked = true;
					break;

			}

			switch (sortColumnB)
			{
				case 0:
					options.sort4.IsChecked = true;
					break;

				case 1:
					options.sort5.IsChecked = true;
					break;

				case 2:
					options.sort6.IsChecked = true;
					break;

				case 3:
					options.sort7.IsChecked = true;
					break;

			}

			options.ShowDialog();

			foreach (ReportColumn column in report.Columns)
			{
				column.Sort = Sort.None;
				column.SortPriority = 0;
			}

			if (options.sort1.IsChecked == true)
			{
				report.Columns[0].Sort = Sort.Ascending;
				report.Columns[0].SortPriority = 2;
			}

			if (options.sort2.IsChecked == true)
			{
				report.Columns[1].Sort = Sort.Ascending;
				report.Columns[1].SortPriority = 2;
			}

			if (options.sort3.IsChecked == true)
			{
				report.Columns[2].Sort = Sort.Ascending;
				report.Columns[2].SortPriority = 2;
			}

			if (options.sort5.IsChecked == true)
			{
				report.Columns[0].Sort = Sort.Ascending;
				report.Columns[0].SortPriority = 1;
			}

			if (options.sort6.IsChecked == true)
			{
				report.Columns[1].Sort = Sort.Ascending;
				report.Columns[1].SortPriority = 1;
			}

			if (options.sort7.IsChecked == true)
			{
				report.Columns[2].Sort = Sort.Ascending;
				report.Columns[2].SortPriority = 1;
			}

			report.Options.GroupType = options.enableGrouping.IsChecked == true ? ReportGroupType.ByContacts : ReportGroupType.None;
			customPaintDuration = options.customPaint.IsChecked == true;
			report.Columns[2].Visible = options.displayDuration.IsChecked == true;
		}


		private MindFusion.Scheduling.Reporting.Report report;
		private bool customPaintDuration = true;
	}
}
